@@ -1,11 +1,10 @@
 #!/usr/bin/python
-# Copyright 2012-2013 Gentoo Foundation
-# Copyright 2012-2013 Mike Frysinger <vapier@gentoo.org>
+# Copyright 2012-2014 Gentoo Foundation
+# Copyright 2012-2014 Mike Frysinger <vapier@gentoo.org>
+# Copyright 2012-2014 The Chromium OS Authors
 # Use of this source code is governed by a BSD-style license (BSD-3)
 # pylint: disable=C0301
-# $Header: /var/cvsroot/gentoo-projects/pax-utils/lddtree.py,v 1.44 2014/03/20 08:25:45 vapier Exp $
-
-# TODO: Handle symlinks.
+# $Header: /var/cvsroot/gentoo-projects/pax-utils/lddtree.py,v 1.56 2014/11/20 01:22:36 vapier Exp $
 
 """Read the ELF dependency tree and show it
 
@@ -28,7 +27,7 @@ from elftools.common import exceptions
 
 def warn(msg, prefix='warning'):
   """Write |msg| to stderr with a |prefix| before it"""
-  print('%s: %s: %s' % (sys.argv[0], prefix, msg), file=sys.stderr)
+  print('%s: %s: %s' % (os.path.basename(sys.argv[0]), prefix, msg), file=sys.stderr)
 
 
 def err(msg, status=1):
@@ -37,6 +36,12 @@ def err(msg, status=1):
   sys.exit(status)
 
 
+def dbg(debug, *args, **kwargs):
+  """Pass |args| and |kwargs| to print() when |debug| is True"""
+  if debug:
+    print(*args, **kwargs)
+
+
 def bstr(buf):
   """Decode the byte string into a string"""
   return buf.decode('utf-8')
@@ -53,6 +58,34 @@ def normpath(path):
   return os.path.normpath(path).replace('//', '/')
 
 
+def readlink(path, root, prefixed=False):
+  """Like os.readlink(), but relative to a |root|
+
+  This does not currently handle the pathological case:
+    /lib/foo.so -> ../../../../../../../foo.so
+  This relies on the .. entries in / to point to itself.
+
+  Args:
+    path: The symlink to read
+    root: The path to use for resolving absolute symlinks
+    prefixed: When False, the |path| must not have |root| prefixed to it, nor
+              will the return value have |root| prefixed.  When True, |path|
+              must have |root| prefixed, and the return value will have |root|
+              added.
+
+  Returns:
+    A fully resolved symlink path
+  """
+  root = root.rstrip('/')
+  if prefixed:
+    path = path[len(root):]
+
+  while os.path.islink(root + path):
+    path = os.path.join(os.path.dirname(path), os.readlink(root + path))
+
+  return normpath((root + path) if prefixed else path)
+
+
 def makedirs(path):
   """Like os.makedirs(), but ignore EEXIST errors"""
   try:
@@ -85,10 +118,10 @@ def GenerateLdsoWrapper(root, path, interp, libpaths=()):
   interp_dir, interp_name = os.path.split(interp)
   libpaths = dedupe([interp_dir] + list(libpaths))
   replacements = {
-    'interp': os.path.join(os.path.relpath(interp_dir, basedir),
-                           interp_name),
-    'libpaths': ':'.join(['${basedir}/' + os.path.relpath(p, basedir)
-                          for p in libpaths]),
+      'interp': os.path.join(os.path.relpath(interp_dir, basedir),
+                             interp_name),
+      'libpaths': ':'.join(['${basedir}/' + os.path.relpath(p, basedir)
+                            for p in libpaths]),
   }
   wrapper = """#!/bin/sh
 if ! base=$(realpath "$0" 2>/dev/null); then
@@ -141,7 +174,7 @@ def ParseLdPaths(str_ldpaths, root='', path=None):
   return dedupe(ldpaths)
 
 
-def ParseLdSoConf(ldso_conf, root='/', _first=True):
+def ParseLdSoConf(ldso_conf, root='/', debug=False, _first=True):
   """Load all the paths from a given ldso config file
 
   This should handle comments, whitespace, and "include" statements.
@@ -149,6 +182,7 @@ def ParseLdSoConf(ldso_conf, root='/', _first=True):
   Args:
     ldso_conf: The file to scan
     root: The path to prepend to all paths found
+    debug: Enable debug output
     _first: Recursive use only; is this the first ELF ?
 
   Returns:
@@ -156,7 +190,9 @@ def ParseLdSoConf(ldso_conf, root='/', _first=True):
   """
   paths = []
 
+  dbg_pfx = '' if _first else '  '
   try:
+    dbg(debug, '%sParseLdSoConf(%s)' % (dbg_pfx, ldso_conf))
     with open(ldso_conf) as f:
       for line in f.readlines():
         line = line.split('#', 1)[0].strip()
@@ -168,8 +204,9 @@ def ParseLdSoConf(ldso_conf, root='/', _first=True):
             line = root + line.lstrip('/')
           else:
             line = os.path.dirname(ldso_conf) + '/' + line
+          dbg(debug, '%s  glob: %s' % (dbg_pfx, line))
           for path in glob.glob(line):
-            paths += ParseLdSoConf(path, root=root, _first=False)
+            paths += ParseLdSoConf(path, root=root, debug=debug, _first=False)
         else:
           paths += [normpath(root + line)]
   except IOError as e:
@@ -184,7 +221,7 @@ def ParseLdSoConf(ldso_conf, root='/', _first=True):
   return paths
 
 
-def LoadLdpaths(root='/', prefix=''):
+def LoadLdpaths(root='/', prefix='', debug=False):
   """Load linker paths from common locations
 
   This parses the ld.so.conf and LD_LIBRARY_PATH env var.
@@ -192,14 +229,15 @@ def LoadLdpaths(root='/', prefix=''):
   Args:
     root: The root tree to prepend to paths
     prefix: The path under |root| to search
+    debug: Enable debug output
 
   Returns:
     dict containing library paths to search
   """
   ldpaths = {
-    'conf': [],
-    'env': [],
-    'interp': [],
+      'conf': [],
+      'env': [],
+      'interp': [],
   }
 
   # Load up $LD_LIBRARY_PATH.
@@ -214,7 +252,8 @@ def LoadLdpaths(root='/', prefix=''):
       ldpaths['env'] = ParseLdPaths(env_ldpath, path='')
 
   # Load up /etc/ld.so.conf.
-  ldpaths['conf'] = ParseLdSoConf(root + prefix + '/etc/ld.so.conf', root=root)
+  ldpaths['conf'] = ParseLdSoConf(root + prefix + '/etc/ld.so.conf', root=root,
+                                  debug=debug)
 
   return ldpaths
 
@@ -234,37 +273,48 @@ def CompatibleELFs(elf1, elf2):
   """
   osabis = frozenset([e.header['e_ident']['EI_OSABI'] for e in (elf1, elf2)])
   compat_sets = (
-    frozenset('ELFOSABI_%s' % x for x in ('NONE', 'SYSV', 'GNU', 'LINUX',)),
+      frozenset('ELFOSABI_%s' % x for x in ('NONE', 'SYSV', 'GNU', 'LINUX',)),
   )
   return ((len(osabis) == 1 or any(osabis.issubset(x) for x in compat_sets)) and
-    elf1.elfclass == elf2.elfclass and
-    elf1.little_endian == elf2.little_endian and
-    elf1.header['e_machine'] == elf2.header['e_machine'])
+          elf1.elfclass == elf2.elfclass and
+          elf1.little_endian == elf2.little_endian and
+          elf1.header['e_machine'] == elf2.header['e_machine'])
 
 
-def FindLib(elf, lib, ldpaths):
+def FindLib(elf, lib, ldpaths, root='/', debug=False):
   """Try to locate a |lib| that is compatible to |elf| in the given |ldpaths|
 
   Args:
-    elf: the elf which the library should be compatible with (ELF wise)
-    lib: the library (basename) to search for
-    ldpaths: a list of paths to search
+    elf: The elf which the library should be compatible with (ELF wise)
+    lib: The library (basename) to search for
+    ldpaths: A list of paths to search
+    root: The root path to resolve symlinks
+    debug: Enable debug output
 
   Returns:
-    the full path to the desired library
+    Tuple of the full path to the desired library and the real path to it
   """
+  dbg(debug, '  FindLib(%s)' % lib)
+
   for ldpath in ldpaths:
     path = os.path.join(ldpath, lib)
-    if os.path.exists(path):
-      with open(path, 'rb') as f:
+    target = readlink(path, root, prefixed=True)
+    if path != target:
+      dbg(debug, '    checking: %s -> %s' % (path, target))
+    else:
+      dbg(debug, '    checking:', path)
+
+    if os.path.exists(target):
+      with open(target, 'rb') as f:
         libelf = ELFFile(f)
         if CompatibleELFs(elf, libelf):
-          return path
-  return None
+          return (target, path)
+
+  return (None, None)
 
 
 def ParseELF(path, root='/', prefix='', ldpaths={'conf':[], 'env':[], 'interp':[]},
-             _first=True, _all_libs={}):
+             display=None, debug=False, _first=True, _all_libs={}):
   """Parse the ELF dependency tree of the specified file
 
   Args:
@@ -274,6 +324,8 @@ def ParseELF(path, root='/', prefix='', ldpaths={'conf':[], 'env':[], 'interp':[
     prefix: The path under |root| to search
     ldpaths: dict containing library paths to search; should have the keys:
              conf, env, interp
+    display: The path to show rather than |path|
+    debug: Enable debug output
     _first: Recursive use only; is this the first ELF ?
     _all_libs: Recursive use only; dict of all libs we've seen
 
@@ -298,14 +350,17 @@ def ParseELF(path, root='/', prefix='', ldpaths={'conf':[], 'env':[], 'interp':[
     _all_libs = {}
     ldpaths = ldpaths.copy()
   ret = {
-    'interp': None,
-    'path': path,
-    'needed': [],
-    'rpath': [],
-    'runpath': [],
-    'libs': _all_libs,
+      'interp': None,
+      'path': path if display is None else display,
+      'realpath': path,
+      'needed': [],
+      'rpath': [],
+      'runpath': [],
+      'libs': _all_libs,
   }
 
+  dbg(debug, 'ParseELF(%s)' % path)
+
   with open(path, 'rb') as f:
     elf = ELFFile(f)
 
@@ -316,16 +371,19 @@ def ParseELF(path, root='/', prefix='', ldpaths={'conf':[], 'env':[], 'interp':[
           continue
 
         interp = bstr(segment.get_interp_name())
+        dbg(debug, '  interp           =', interp)
         ret['interp'] = normpath(root + interp)
         ret['libs'][os.path.basename(interp)] = {
-          'path': ret['interp'],
-          'needed': [],
+            'path': ret['interp'],
+            'realpath': readlink(ret['interp'], root, prefixed=True),
+            'needed': [],
         }
         # XXX: Should read it and scan for /lib paths.
         ldpaths['interp'] = [
-          normpath(root + os.path.dirname(interp)),
-          normpath(root + prefix + '/usr' + os.path.dirname(interp).lstrip(prefix)),
+            normpath(root + os.path.dirname(interp)),
+            normpath(root + prefix + '/usr' + os.path.dirname(interp).lstrip(prefix)),
         ]
+        dbg(debug, '  ldpaths[interp]  =', ldpaths['interp'])
         break
 
     # Parse the ELF's dynamic tags.
@@ -355,6 +413,8 @@ def ParseELF(path, root='/', prefix='', ldpaths={'conf':[], 'env':[], 'interp':[
       # used at runtime to locate things.
       ldpaths['rpath'] = rpaths
       ldpaths['runpath'] = runpaths
+      dbg(debug, '  ldpaths[rpath]   =', rpaths)
+      dbg(debug, '  ldpaths[runpath] =', runpaths)
     ret['rpath'] = rpaths
     ret['runpath'] = runpaths
     ret['needed'] = libs
@@ -366,13 +426,15 @@ def ParseELF(path, root='/', prefix='', ldpaths={'conf':[], 'env':[], 'interp':[
         continue
       if all_ldpaths is None:
         all_ldpaths = rpaths + ldpaths['rpath'] + ldpaths['env'] + runpaths + ldpaths['runpath'] + ldpaths['conf'] + ldpaths['interp']
-      fullpath = FindLib(elf, lib, all_ldpaths)
+      realpath, fullpath = FindLib(elf, lib, all_ldpaths, root, debug=debug)
       _all_libs[lib] = {
-        'path': fullpath,
-        'needed': [],
+          'realpath': realpath,
+          'path': fullpath,
+          'needed': [],
       }
       if fullpath:
-        lret = ParseELF(fullpath, root, prefix, ldpaths, False, _all_libs)
+        lret = ParseELF(realpath, root, prefix, ldpaths, display=fullpath,
+                        debug=debug, _first=False, _all_libs=_all_libs)
         _all_libs[lib]['needed'] = lret['needed']
 
     del elf
@@ -385,7 +447,7 @@ def _NormalizePath(option, _opt, value, parser):
 
 
 def _ShowVersion(_option, _opt, _value, _parser):
-  d = '$Id: lddtree.py,v 1.44 2014/03/20 08:25:45 vapier Exp $'.split()
+  d = '$Id: lddtree.py,v 1.56 2014/11/20 01:22:36 vapier Exp $'.split()
   print('%s-%s %s %s' % (d[1].split('.')[0], d[2], d[3], d[4]))
   sys.exit(0)
 
@@ -434,8 +496,9 @@ def _ActionCopy(options, elf):
   def _StripRoot(path):
     return path[len(options.root) - 1:]
 
-  def _copy(src, striproot=True, wrapit=False, libpaths=(), outdir=None):
-    if src is None:
+  def _copy(realsrc, src, striproot=True, wrapit=False, libpaths=(),
+            outdir=None):
+    if realsrc is None:
       return
 
     if wrapit:
@@ -454,7 +517,7 @@ def _ActionCopy(options, elf):
     try:
       # See if they're the same file.
       nstat = os.stat(dst + ('.elf' if wrapit else ''))
-      ostat = os.stat(src)
+      ostat = os.stat(realsrc)
       for field in ('mode', 'mtime', 'size'):
         if getattr(ostat, 'st_' + field) != \
            getattr(nstat, 'st_' + field):
@@ -470,10 +533,10 @@ def _ActionCopy(options, elf):
 
     makedirs(os.path.dirname(dst))
     try:
-      shutil.copy2(src, dst)
+      shutil.copy2(realsrc, dst)
     except IOError:
       os.unlink(dst)
-      shutil.copy2(src, dst)
+      shutil.copy2(realsrc, dst)
 
     if wrapit:
       if options.verbose:
@@ -492,10 +555,14 @@ def _ActionCopy(options, elf):
   # for known libs that get loaded (e.g. curl will dlopen(libresolv)).
   libpaths = set()
   for lib in elf['libs']:
-    path = elf['libs'][lib]['path']
+    libdata = elf['libs'][lib]
+    path = libdata['realpath']
+    if path is None:
+      warn('could not locate library: %s' % lib)
+      continue
     if not options.libdir:
       libpaths.add(_StripRoot(os.path.dirname(path)))
-    _copy(path, outdir=options.libdir)
+    _copy(path, libdata['path'], outdir=options.libdir)
 
   if not options.libdir:
     libpaths = list(libpaths)
@@ -506,8 +573,10 @@ def _ActionCopy(options, elf):
   else:
     libpaths.add(options.libdir)
 
-  _copy(elf['interp'], outdir=options.libdir)
-  _copy(elf['path'], striproot=options.auto_root,
+  # We don't bother to copy this as ParseElf adds the interp to the 'libs',
+  # so it was already copied in the libs loop above.
+  #_copy(elf['interp'], outdir=options.libdir)
+  _copy(elf['realpath'], elf['path'], striproot=options.auto_root,
         wrapit=options.generate_wrappers, libpaths=libpaths,
         outdir=options.bindir)
 
@@ -546,54 +615,55 @@ collapse all that down into nice directory structure.
 This will place bash, lspci, and lsof into /foo/bin/.  All the libraries
 they need will be placed into /foo/lib/ only.""")
   parser.add_option('-a', '--all',
-    action='store_true', default=False,
-    help='Show all duplicated dependencies')
+                    action='store_true', default=False,
+                    help='Show all duplicated dependencies')
   parser.add_option('-R', '--root',
-    default=os.environ.get('ROOT', ''), type='string',
-    action='callback', callback=_NormalizePath,
-    help='Search for all files/dependencies in ROOT')
+                    default=os.environ.get('ROOT', ''), type='string',
+                    action='callback', callback=_NormalizePath,
+                    help='Search for all files/dependencies in ROOT')
   parser.add_option('-P', '--prefix',
-    default=os.environ.get('EPREFIX', '@GENTOO_PORTAGE_EPREFIX@'), type='string',
-    action='callback', callback=_NormalizePath,
-    help='Specify EPREFIX for binaries (for Gentoo Prefix)')
+                    default=os.environ.get(
+                        'EPREFIX', '@GENTOO_PORTAGE_EPREFIX@'), type='string',
+                    action='callback', callback=_NormalizePath,
+                    help='Specify EPREFIX for binaries (for Gentoo Prefix)')
   parser.add_option('--no-auto-root',
-    dest='auto_root', action='store_false', default=True,
-    help='Do not automatically prefix input ELFs with ROOT')
+                    dest='auto_root', action='store_false', default=True,
+                    help='Do not automatically prefix input ELFs with ROOT')
   parser.add_option('-l', '--list',
-    action='store_true', default=False,
-    help='Display output in a simple list (easy for copying)')
+                    action='store_true', default=False,
+                    help='Display output in a simple list (easy for copying)')
   parser.add_option('-x', '--debug',
-    action='store_true', default=False,
-    help='Run with debugging')
+                    action='store_true', default=False,
+                    help='Run with debugging')
   parser.add_option('-v', '--verbose',
-    action='store_true', default=False,
-    help='Be verbose')
+                    action='store_true', default=False,
+                    help='Be verbose')
   parser.add_option('--skip-non-elfs',
-    action='store_true', default=False,
-    help='Skip plain (non-ELF) files instead of warning')
+                    action='store_true', default=False,
+                    help='Skip plain (non-ELF) files instead of warning')
   parser.add_option('-V', '--version',
-    action='callback', callback=_ShowVersion,
-    help='Show version information')
+                    action='callback', callback=_ShowVersion,
+                    help='Show version information')
 
   group = optparse.OptionGroup(parser, 'Copying options')
   group.add_option('--copy-to-tree',
-    dest='dest', default=None, type='string',
-    action='callback', callback=_NormalizePath,
-    help='Copy all files to the specified tree')
+                   dest='dest', default=None, type='string',
+                   action='callback', callback=_NormalizePath,
+                   help='Copy all files to the specified tree')
   group.add_option('--bindir',
-    default=None, type='string',
-    action='callback', callback=_NormalizePath,
-    help='Dir to store all ELFs specified on the command line')
+                   default=None, type='string',
+                   action='callback', callback=_NormalizePath,
+                   help='Dir to store all ELFs specified on the command line')
   group.add_option('--libdir',
-    default=None, type='string',
-    action='callback', callback=_NormalizePath,
-    help='Dir to store all ELF libs')
+                   default=None, type='string',
+                   action='callback', callback=_NormalizePath,
+                   help='Dir to store all ELF libs')
   group.add_option('--generate-wrappers',
-    action='store_true', default=False,
-    help='Wrap executable ELFs with scripts for local ldso')
+                   action='store_true', default=False,
+                   help='Wrap executable ELFs with scripts for local ldso')
   group.add_option('--copy-non-elfs',
-    action='store_true', default=False,
-    help='Copy over plain (non-ELF) files instead of warn+ignore')
+                   action='store_true', default=False,
+                   help='Copy over plain (non-ELF) files instead of warn+ignore')
   parser.add_option_group(group)
 
   (options, paths) = parser.parse_args(argv)
@@ -611,32 +681,49 @@ they need will be placed into /foo/lib/ only.""")
   if options.skip_non_elfs and options.copy_non_elfs:
     parser.error('pick one handler for non-ELFs: skip or copy')
 
-  if options.debug:
-    print('root =', options.root)
-    if options.dest:
-      print('dest =', options.dest)
+  dbg(options.debug, 'root =', options.root)
+  if options.dest:
+    dbg(options.debug, 'dest =', options.dest)
   if not paths:
     err('missing ELF files to scan')
 
-  ldpaths = LoadLdpaths(options.root, options.prefix)
-  if options.debug:
-    print('ldpaths[conf] =', ldpaths['conf'])
-    print('ldpaths[env]  =', ldpaths['env'])
+  ldpaths = LoadLdpaths(options.root, options.prefix, debug=options.debug)
+  dbg(options.debug, 'ldpaths[conf] =', ldpaths['conf'])
+  dbg(options.debug, 'ldpaths[env]  =', ldpaths['env'])
 
   # Process all the files specified.
   ret = 0
   for path in paths:
+    dbg(options.debug, 'argv[x]       =', path)
     # Only auto-prefix the path if the ELF is absolute.
     # If it's a relative path, the user most likely wants
     # the local path.
     if options.auto_root and path.startswith('/'):
       path = options.root + path.lstrip('/')
+      dbg(options.debug, '  +auto-root  =', path)
 
     matched = False
     for p in glob.iglob(path):
+      # Once we've processed the globs, resolve the symlink.  This way you can
+      # operate on a path that is an absolute symlink itself.  e.g.:
+      #   $ ln -sf /bin/bash $PWD/root/bin/sh
+      #   $ lddtree --root $PWD/root /bin/sh
+      # First we'd turn /bin/sh into $PWD/root/bin/sh, then we want to resolve
+      # the symlink to $PWD/root/bin/bash rather than a plain /bin/bash.
+      dbg(options.debug, '  globbed     =', p)
+      if not path.startswith('/'):
+        realpath = os.path.realpath(path)
+      elif options.auto_root:
+        realpath = readlink(p, options.root, prefixed=True)
+      else:
+        realpath = path
+      if path != realpath:
+        dbg(options.debug, '  resolved    =', realpath)
+
       matched = True
       try:
-        elf = ParseELF(p, options.root, options.prefix, ldpaths)
+        elf = ParseELF(realpath, options.root, options.prefix, ldpaths,
+                       display=p, debug=options.debug)
       except exceptions.ELFError as e:
         if options.skip_non_elfs:
           continue
@@ -644,11 +731,12 @@ they need will be placed into /foo/lib/ only.""")
         if options.dest is not None and options.copy_non_elfs:
           if os.path.exists(p):
             elf = {
-              'interp': None,
-              'libs': [],
-              'runpath': [],
-              'rpath': [],
-              'path': p,
+                'interp': None,
+                'libs': [],
+                'runpath': [],
+                'rpath': [],
+                'path': p,
+                'realpath': realpath,
             }
             _ActionCopy(options, elf)
             continue
